"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApi = void 0;
const axios_1 = __importDefault(require("axios"));
const uuid_1 = require("uuid");
const logger_1 = require("../logger");
function createApi(axiosConfig) {
    const axiosApi = axios_1.default.create(axiosConfig);
    axiosApi.interceptors.request.use((request) => {
        if (request === null || request === void 0 ? void 0 : request.logRequest) {
            const { data, params, method } = request;
            const fullUrl = getFullUrlFromConfig(request);
            request.meta = request.meta || {};
            request.meta.requestId = uuid_1.v4();
            request.meta.requestStartedAt = new Date().getTime();
            logRequest(request.meta.requestId, method, fullUrl, data, params);
        }
        return request;
    });
    axiosApi.interceptors.response.use((response) => {
        var _a;
        if ((_a = response.config) === null || _a === void 0 ? void 0 : _a.logResponse) {
            const { status, data, config } = response;
            const { method, meta } = config;
            const fullUrl = getFullUrlFromConfig(config);
            const responseTime = getResponseTimeFromConfig(config);
            logResponse(meta === null || meta === void 0 ? void 0 : meta.requestId, method, fullUrl, data, status, responseTime);
        }
        return response;
    }, (error) => {
        const { config, message, response } = error;
        const { method, meta } = config;
        const fullUrl = getFullUrlFromConfig(config);
        logError(meta === null || meta === void 0 ? void 0 : meta.requestId, method, fullUrl, response === null || response === void 0 ? void 0 : response.data, response === null || response === void 0 ? void 0 : response.status, message);
        throw error;
    });
    return axiosApi;
}
exports.createApi = createApi;
function logRequest(id, method, url, body, params) {
    const request = {
        ['context-id']: id,
        url,
        method: method === null || method === void 0 ? void 0 : method.toUpperCase(),
        params,
        body
    };
    logger_1.logger.info('sm-utility/api-request', { request });
}
function logResponse(id, method, url, body, status, responseTime) {
    const response = {
        ['context-id']: id,
        url,
        method: method === null || method === void 0 ? void 0 : method.toUpperCase(),
        body,
        status,
        responseTime
    };
    logger_1.logger.info('sm-utility/api-response', { response });
}
function logError(id, method, url, data, status, message) {
    const err = {
        ['context-id']: id,
        url,
        method: method === null || method === void 0 ? void 0 : method.toUpperCase(),
        data,
        status,
        message
    };
    logger_1.logger.error('sm-utility/api-error', { err });
}
function getResponseTimeFromConfig(config) {
    var _a;
    const requestStart = (_a = config === null || config === void 0 ? void 0 : config.meta) === null || _a === void 0 ? void 0 : _a.requestStartedAt;
    let responseTime = '-';
    if (requestStart) {
        responseTime = String(new Date().getTime() - requestStart);
    }
    return responseTime + ' ms';
}
function getFullUrlFromConfig(config) {
    const { baseURL, url } = config;
    const fullUrl = baseURL && url ? new String().concat(baseURL, url) : '';
    return fullUrl;
}
