declare type GenericClass<T> = (new (...args: any[]) => T);
declare type UnwrapPromise<T> = T extends Promise<infer U> ? U : T;
declare type TypesByMethodNames<T> = {
    [K in keyof T]?: T[K] extends (...args: any[]) => infer R ? R : never;
};
declare type ResolvedTypesByMethodNames<T> = {
    [K in keyof T]?: T[K] extends (...args: any[]) => infer R ? UnwrapPromise<R> : never;
};
/**
 * Creates a mock instance of a given class with customized method implementations,
 * where method return values are automatically unwrapped if they are promises.
 * @example
 * const userRepository = mockResolvedClass(UserRepository, {
 *   findUserByToken: { token: 'adhf753', name: 'John' },
 * });
 * @param Class - The class to mock.
 * @param givenMethods - An object containing customized key-value pairs where the key
 * is the method name and the value is the resolved method value.
 * @returns A mocked instance of the provided class with overridden methods.
 */
export declare function mockResolvedClass<T>(Class: GenericClass<T>, givenMethods?: ResolvedTypesByMethodNames<T>): T;
/**
 * Creates a mock instance of a given class with customized method implementations.
 * @example
 * const userRepository = mockClass(UserRepository, {
 *   findUserByToken: Promise.resolve({ token: 'adhf753', name: 'John' }),
 * });
 * @param Class - The class to mock.
 * @param methods - An object containing customized key-value pairs where the key
 * is the method name and the value is the returned method value.
 * @returns A mocked instance of the provided class with overridden methods.
 */
export declare function mockClass<T>(Class: GenericClass<T>, givenMethods?: TypesByMethodNames<T>): T;
/**
 * Creates a raw mock instance of a given class with no method implementations.
 * This function is intended for use with testing frameworks like jest (spyOn) when
 * you want to set up **manual** method spies. Alternatively, consider using
 * {@link mockClass} or {@link mockResolvedClass} for more comprehensive method mocking.
 * @example
 * const rawUserRepository = rawInstance(UserRepository);
 * jest.spyOn(rawUserRepository, 'findUserByToken').mockRejectedValue(new Error('Token expired'));
 * @param Class - The class to mock.
 * @returns A raw mocked instance of the provided class with no implemented methods.
 */
export declare function rawInstance<T>(Class: GenericClass<T>): T;
export {};
